import { assignDeepOmitBlank, ChartMeasureRoleType, FilteringLogic, getChartCategory, getChartSeries, getEntityHierarchy, getEntityProperty, getPropertyCaption, getPropertyHierarchy, mergeOptions, omitBlank } from '@metad/ocap-core';
import { assign, isEmpty, isNil, uniqBy } from 'lodash-es';
import { getMeasureAxis } from './axis';
import { dimensionToSeriesComponent, getCoordinateSystem, serializeSeriesComponent } from './cartesian';
import { formatMeasureNumber } from './common';
import { coordinates, gatherCoordinates } from './coordinates';
import { referenceLines } from './series';
import { getEChartsTooltip } from './tooltip';
import { AxisEnum } from './types';
export function scatter(data, chartAnnotation, entityType, settings, options) {
    const context = {
        data,
        entityType,
        settings,
        options,
        chartAnnotation
    };
    const pieCoordinates = coordinates(context, 'grid', scatterCoordinate);
    return {
        ...context,
        echartsOptions: gatherCoordinates(pieCoordinates, 'scatter', options),
        onClick: (event) => {
            const dimension = chartAnnotation.dimensions[0];
            const hierarchy = getEntityHierarchy(entityType, dimension);
            return {
                ...event,
                event: event.event?.event,
                filter: chartAnnotation.dimensions.length > 1 ? {
                    filteringLogic: FilteringLogic.And,
                    children: chartAnnotation.dimensions.map((dimension) => {
                        const hierarchy = getEntityHierarchy(entityType, dimension);
                        return {
                            dimension,
                            members: [
                                {
                                    value: event.data[hierarchy.name],
                                    label: event.data[getPropertyCaption(hierarchy)],
                                }
                            ]
                        };
                    })
                } : {
                    dimension,
                    members: [
                        {
                            value: event.data[hierarchy.name],
                            label: event.data[getPropertyCaption(hierarchy)],
                        }
                    ]
                }
            };
        }
    };
}
/**
 * 单个散点图笛卡尔坐标系, 对应一个 grid 内的组件
 *
 * 包含对 dimension role 为 series 的处理
 *
 * @param data 数据数组
 * @param chartAnnotation 图形注解
 * @param entityType 实体类型
 * @param settings 图形设置
 * @param options 图形属性
 * @returns
 */
export function scatterCoordinate(context, data) {
    const { chartAnnotation, entityType, settings, options } = context;
    const category = getChartCategory(chartAnnotation);
    const categoryProperty = getEntityProperty(entityType, category);
    const chartSeries = getChartSeries(chartAnnotation);
    // @todo 抽象成同一的 Axis 类型以支持 Dimension 和 Measure
    const axis1 = getMeasureAxis(chartAnnotation, ChartMeasureRoleType.Axis1, 0);
    const axis2 = getMeasureAxis(chartAnnotation, ChartMeasureRoleType.Axis2, 1);
    context.datasets = [];
    if (chartSeries) {
        const chartSeriesName = getPropertyHierarchy(chartSeries);
        const chartSeriesTextName = getPropertyCaption(getEntityProperty(entityType, chartSeries));
        const categoryValues = uniqBy(data, chartSeriesName).map((x) => ({
            value: x[chartSeriesName],
            label: x[chartSeriesTextName]
        }));
        const dataset = [
            {
                source: data
            }
        ];
        const seriesComponents = [];
        // Split the dataset into groups by transform filter
        categoryValues.forEach(({ value, label }, index) => {
            dataset.push({
                transform: [
                    {
                        type: 'filter',
                        config: { dimension: chartSeriesName, value: value }
                    }
                ]
            });
            const seriesComponent = dimensionToSeriesComponent(label || value, chartSeries, chartAnnotation.measures, data, entityType, settings);
            seriesComponent.datasetIndex = index + 1;
            seriesComponents.push(seriesComponent);
        });
        context.datasets.push({
            dataset,
            seriesComponents,
            tooltip: {
                valueFormatter: (value) => {
                    return [
                        formatMeasureNumber({ measure: axis1, property: null }, value[0], settings.locale, axis1.formatting?.shortNumber),
                        formatMeasureNumber({ measure: axis2, property: null }, value[1], settings.locale, axis2.formatting?.shortNumber)
                    ];
                }
            }
        });
    }
    else {
        const seriesComponent = dimensionToSeriesComponent(null, category, chartAnnotation.measures, data, entityType, settings);
        context.datasets = [
            {
                dataset: {
                    source: data
                },
                seriesComponents: [seriesComponent],
                tooltip: getEChartsTooltip(options?.tooltip, categoryProperty, chartAnnotation.measures.map((measure) => ({
                    measure,
                    property: getEntityProperty(entityType, measure)
                })), [seriesComponent], settings.locale)
            }
        ];
    }
    const { categoryAxis, valueAxis } = getCoordinateSystem(context, data, settings.locale);
    const gridOptions = {
        type: 'Cartesian2d',
        name: '',
        grid: assignDeepOmitBlank({
            top: 0,
            right: 0,
            bottom: 0,
            left: 0,
            containLabel: true
        }, options?.grid),
        [categoryAxis.orient]: categoryAxis.axis,
        [valueAxis.orient]: valueAxis.axis,
        // visualMap: [],
        datasets: [],
        tooltip: [],
        legend: [],
        dataZoom: []
    };
    if (categoryAxis.dataZoom) {
        gridOptions.dataZoom.push(categoryAxis.dataZoom);
    }
    if (valueAxis.dataZoom) {
        gridOptions.dataZoom.push(valueAxis.dataZoom);
    }
    context.datasets.forEach(({ dataset, seriesComponents, tooltip }) => {
        gridOptions.datasets.push({
            dataset,
            series: seriesComponents.map((seriesComponent) => {
                const { series, visualMaps } = serializeSeriesComponent(dataset, seriesComponent, entityType, valueAxis, settings, options);
                // gridOptions.visualMap.push(...visualMaps)
                if (!isNil(seriesComponent.valueAxisIndex)) {
                    series[valueAxis.orient + 'Index'] = seriesComponent.valueAxisIndex;
                }
                /**
                 * 堆积系列情况下不适用 encode, 使用的应该是 seriesLayoutBy: "row" name: "[Canada]" 与 dataset 中的行数据进行的对应
                 */
                if (isNil(series.seriesLayoutBy)) {
                    series.encode = {
                        [AxisEnum[categoryAxis.orient]]: axis1?.measure,
                        [AxisEnum[valueAxis.orient]]: axis2?.measure,
                        itemName: getPropertyCaption(categoryProperty),
                        tooltip: [axis1?.measure, axis2?.measure, ...seriesComponent.tooltip]
                    };
                }
                /**
                 * 依赖于 Measure 1 和 Measure 2 的 referenceLines
                 */
                if (!isEmpty(axis1?.referenceLines) || !isEmpty(axis2?.referenceLines)) {
                    series.markLine = mergeOptions({ data: [] }, options.seriesStyle.markLine);
                    assign(series, referenceLines(axis1, axis2, options, valueAxis.orient, settings?.locale));
                }
                series.tooltip = tooltip;
                mergeOptions(series, options?.seriesStyle);
                return {
                    options: series,
                    visualMaps
                };
            })
        });
    });
    if (options?.tooltip?.trigger === 'axis') {
        const measure = getMeasureAxis(chartAnnotation, ChartMeasureRoleType.Axis1, 0);
        gridOptions.tooltip.push({
            ...omitBlank(options.tooltip),
            trigger: 'axis',
            valueFormatter: (value) => {
                return value.map((value) => formatMeasureNumber({ measure, property: null }, value, settings.locale, measure.formatting?.shortNumber));
            }
        });
    }
    else {
        gridOptions.tooltip.push({
            ...omitBlank(options?.tooltip),
            trigger: 'item'
        });
    }
    // legend
    if (options?.legend) {
        gridOptions.legend.push(omitBlank(options.legend));
    }
    return gridOptions;
}
//# sourceMappingURL=scatter.js.map